#! /usr/bin/env perl

use CGI;

use strict;
use warnings;

sub RenderCell($$$) {
	my ($kind, $state, $tag) = @_;

	if ($kind eq 'host') {
		print '<th class="' . $state . '">';
	} else {
		print '<td class="' . $state . '">';
	}
	print $tag;
	print '<br />';
	print '<form method="post" action="index.cgi">';
	print '<input type="hidden" name="kind" value="' . $kind . '" />';
	print '<input type="hidden" name="tag" value="' . $tag . '" />';
	if ($state ne 'new') {
		# XXX Need to check kernel+world states.
		print '<input type="submit" name="action" value="build" />';
	}
	if ($kind eq 'host') {
		print '<input type="submit" name="action" value="deploy" />';
	}
	print '</form>';
	if ($kind eq 'host') {
		print '</th>';
	} else {
		print '</td>';
	}
}

sub Run($) {
	my ($command) = @_;
	my ($response);

	$ENV{'HOME'} = '/usr/people/juli';
	$ENV{'PATH'} = '/sbin:/bin:/usr/sbin:/usr/bin';

	open($response, '/bin/sh ' . $ENV{'HOME'} . '/buildomat/bom.sh ' . $command . ' |');
	return $response;
}

sub ShowTime($) {
	my ($time) = @_;
	my $str;

	if ($time == 0) {
		return 'instantaneous';
	}

	if ($time >= 60 * 60) {
		$str .= int($time / (60 * 60)) . 'h';
		$time %= 60 * 60;
	}
	if ($time >= 60) {
		$str .= int($time / 60) . 'm';
		$time %= 60;
	}
	if ($time > 0) {
		$str .= int($time) . 's';
	}

	return $str;
}

sub Main($) {
	my ($cgi) = @_;

	if ($cgi->param('action')) {
		my $action = $cgi->param('action');
		my $kind = $cgi->param('kind');
		my $tag = $cgi->param('tag');

		my $response = Run('jobsubmit ' . $action . ' -' . $kind . ' ' . $tag);
		while (<$response>) {
		}
		close($response);

		print $cgi->redirect('index.cgi');
		return;
	}

	my (%host_map, %kernel_map, %world_map);
	my ($host, $kind, $tag);

	my $listing = Run('list -a');
	while (<$listing>) {
		chomp;

		my ($key, $value) = m/^([^ ]+) (.+)$/;

		if ($key eq 'host' or $key eq 'kernel' or $key eq 'world') {
			$kind = $key;
			$tag = $value;

			if ($kind eq 'host') {
				$host = $value;
			}
			$host_map{$host}{$kind} = $tag;
			next;
		}

		if ($key eq 'state') {
			if ($kind eq 'host') {
				$host_map{$tag}{$key} = $value;
				next;
			}

			if ($kind eq 'kernel') {
				$kernel_map{$tag}{$key} = $value;
				next;
			}

			if ($kind eq 'world') {
				$world_map{$tag}{$key} = $value;
				next;
			}
		}
	}
	close($listing);

	print $cgi->header('text/html');
	print '<html>';
	print '<head>';
	print '<title>buildomat!</title>';
	print '<link href="style.css" rel="stylesheet" type="text/css" />';
	print '</head>';
	print '<body>';
	print '<h1>buildomat!</h1>';

	print '<h2>hosts</h2>';
	print '<table>';
	print '<tr>';
	print '<th>Host</th>';
	print '<th>Kernel</th>';
	print '<th>World</th>';
	print '</tr>';
	foreach my $host (sort keys %host_map) {
		print '<tr>';
		foreach my $kind ('host', 'kernel', 'world') {
			my $tag = $host_map{$host}{$kind};
			my $state;

			if ($kind eq 'host') {
				$state = $host_map{$tag}{'state'};
			}

			if ($kind eq 'kernel') {
				$state = $kernel_map{$tag}{'state'};
			}

			if ($kind eq 'world') {
				$state = $world_map{$tag}{'state'};
			}

			unless (-e $state) {
				RenderCell($kind, 'stateless', $tag);
				next;
			}

			my ($file, $line);
			open($file, '<' . $state);
			while (<$file>) {
				chomp;
				$line = $_;
			}
			$state = $line;
			close($file);

			unless ($state) {
				RenderCell($kind, 'stateless', $tag);
				next;
			}

			$state =~ s/:\d+$//;
			RenderCell($kind, $state, $tag);
		}

		print '</tr>';
	}
	print '</table>';

	print '<h2>past times</h2>';
	print '<table>';
	print '<tr>';
	print '<th>Kind</th>';
	print '<th>Tag</th>';
	print '<th>Time</th>';
	print '</tr>';
	foreach my $kind ('host', 'kernel', 'world') {
		my %map;

		if ($kind eq 'host') {
			%map = %host_map;
		}

		if ($kind eq 'kernel') {
			%map = %kernel_map;
		}

		if ($kind eq 'world') {
			%map = %world_map;
		}

		foreach my $tag (sort keys %map) {
			my $state = $map{$tag}{'state'};
			unless (-e $state) {
				next;
			}

			my ($last_new, $last_state, $last_time);
			my ($file, $line);
			open($file, '<' . $state);
			while (<$file>) {
				chomp;

				my ($state, $time) = m/^([^:]+):(\d+)$/;

				$last_state = $state;
				$last_time = $time;

				if ($state eq 'new') {
					$last_new = $time;
				}
			}
			$state = $line;
			close($file);

			unless ($last_state) {
				next;
			}
			if ($last_state eq 'new') {
				next;
			}

			print '<tr>';
			print '<td class="' . $last_state . '">' . $kind . '</td>';
			print '<td class="' . $last_state . '">' . $tag. '</td>';
			print '<td class="' . $last_state . '">' . ShowTime($last_time - $last_new) . '</td>';
			print '</tr>';
		}
	}
	print '</table>';

	print '<h2>jobs</h2>';
	print '<p>New jobs are launched every 10 minutes.  XXX Independently of whether jobs are already running.</p>';
	$listing = Run('joblist');
	print '<table>';
	print '<tr>';
	print '<th>Job</th>';
	print '<th>Status</th>';
	print '<th>Command</th>';
	print '</tr>';
	while (<$listing>) {
		chomp;

		my ($state, $request) = m/^([^ ]+) (.+)$/;

		my ($job) = $request =~ m/^.+\/(.+)\.req$/;

		print '<tr>';
		print '<td>' . $job . '</td>';
		print '<td>' . $state . '</td>';

		my ($file);
		open($file, '<' . $request);
		print '<td>';
		while (<$file>) {
			print;
		}
		print '</td>';
		close($file);

		print '</tr>';
	}
	print '</table>';

	print '<p><cite>Copyright &copy; 2010 Redacted Consulting.  All rights reserved.</cite></p>';
	print '</body>';
	print '</html>';
}

my $cgi = new CGI;
Main($cgi);
